<?php

namespace Drupal\search_api_decoupled_ui\Form;

use Drupal\Component\Utility\Unicode;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\search_api_decoupled\Form\SearchApiEndpointForm;
use Drupal\search_api_decoupled_ui\ConfigurableSearchUiElementInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class SearchApiEndpointEditForm.
 *
 * @package Drupal\search_api_decoupled_ui\Form
 */
class SearchApiEndpointUiElementsForm extends SearchApiEndpointForm {

  /**
   * The layout plugin manager.
   *
   * @var \Drupal\Core\Layout\LayoutPluginManagerInterface
   */
  protected $layoutManager;

  /**
   * The search ui element manager.
   *
   * @var \Drupal\search_api_decoupled_ui\SearchApiDecoupledUiElementManager
   */
  protected $searchUiElementManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->layoutManager = $container->get('plugin.manager.core.layout');
    $instance->searchUiElementManager = $container->get('plugin.manager.search_api_decoupled.ui_element_manager');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);
    $user_input = $form_state->getUserInput();
    $form['#title'] = $this->t('UI elements for %name endpoint', ['%name' => $this->entity->label()]);
    $form['#tree'] = TRUE;
    foreach ($form as $key => $element) {
      if (is_array($element) && !empty($element['#type'])) {
        $form[$key]['#access'] = FALSE;
      }
    }
    $layouts = $this->layoutManager->getGroupedDefinitions();
    $available_layouts = [];
    if (!empty($layouts['Search'])) {
      foreach ($layouts['Search'] as $layout_id => $layout) {
        $available_layouts[$layout_id] = $layout->getLabel();
      }
    }
    $form['layout'] = [
      '#title' => $this->t('Choose a layout:'),
      '#type' => 'search_api_decoupled_layout_select',
      '#options' => $available_layouts,
      '#default_value' => $this->entity->getLayout(),
      '#required' => TRUE,
    ];

    // Build the list of existing search ui elements for this search endpoint.
    $form['elements'] = [
      '#type' => 'table',
      '#caption' => $this->t('Search UI elements for this endpoint'),
      '#header' => [
        $this->t('Element'),
        $this->t('Region'),
        $this->t('Weight'),
        $this->t('Operations'),
      ],
      '#tabledrag' => [
        [
          'action' => 'order',
          'relationship' => 'sibling',
          'group' => 'search-ui-element-order-weight',
        ],
      ],
      '#attributes' => [
        'id' => 'search-api-endpoint-ui-elements',
      ],
      '#empty' => $this->t('There are currently no ui elements in this search config. Add one by selecting an option below.'),
      // Render elemens below parent elements.
      '#weight' => 5,
    ];
    /** @var \Drupal\search_api_decoupled_ui\SearchUiElementInterface $element */
    foreach ($this->entity->getElements() as $element) {
      $key = $element->getUuid();
      $form['elements'][$key]['#attributes']['class'][] = 'draggable';
      $form['elements'][$key]['#weight'] = isset($user_input['elements']) ? $user_input['elements'][$key]['weight'] : NULL;
      $form['elements'][$key]['element'] = [
        '#tree' => FALSE,
        'data' => [
          'label' => [
            '#markup' => '<b>' . $element->label() . '.</b> ',
          ],
        ],
      ];
      $regions = $this->entity->getUiElementRegions();
      $form['elements'][$key]['region'] = [
        '#tree' => FALSE,
        'data' => [
          'label' => [
            '#plain_text' => $regions[$element->getRegion()] ?? $regions[$this->entity->getDefaultUiElementRegion()],
          ],
        ],
      ];

      $summary = $element->getSummary();

      if (!empty($summary)) {
        $summary['#prefix'] = ' ';
        $form['elements'][$key]['element']['data']['summary'] = $summary;
      }

      $form['elements'][$key]['weight'] = [
        '#type' => 'weight',
        '#title' => $this->t('Weight for @title', ['@title' => $element->label()]),
        '#title_display' => 'invisible',
        '#default_value' => $element->getWeight(),
        '#attributes' => [
          'class' => ['search-ui-element-order-weight'],
        ],
      ];

      $links = [];
      $is_configurable = $element instanceof ConfigurableSearchUiElementInterface;
      if ($is_configurable) {
        $links['edit'] = [
          'title' => $this->t('Edit'),
          'url' => Url::fromRoute('search_api_decoupled.ui_element_edit_form', [
            'search_api_endpoint' => $this->entity->id(),
            'ui_element' => $key,
          ]),
        ];
      }
      $links['delete'] = [
        'title' => $this->t('Delete'),
        'url' => Url::fromRoute('search_api_decoupled.ui_element_delete', [
          'search_api_endpoint' => $this->entity->id(),
          'ui_element' => $key,
        ]),
      ];
      $form['elements'][$key]['operations'] = [
        '#type' => 'operations',
        '#links' => $links,
      ];
    }

    // Build the new search ui element addition form
    // and add it to the UI elements list.
    $new_element_options = [];
    $elements = $this->searchUiElementManager->getDefinitions();
    uasort($elements, function ($a, $b) {
      return Unicode::strcasecmp($a['label'], $b['label']);
    });
    foreach ($elements as $element => $definition) {
      $new_element_options[$element] = $definition['label'];
    }
    $form['elements']['new'] = [
      '#tree' => FALSE,
      '#weight' => $user_input['weight'] ?? NULL,
      '#attributes' => ['class' => ['draggable']],
    ];
    $form['elements']['new']['element'] = [
      'data' => [
        'new' => [
          '#type' => 'select',
          '#title' => $this->t('Element'),
          '#title_display' => 'invisible',
          '#options' => $new_element_options,
          '#empty_option' => $this->t('Select a new element'),
        ],
        [
          'add' => [
            '#type' => 'submit',
            '#value' => $this->t('Add'),
            '#validate' => ['::elementValidate'],
            '#submit' => ['::submitForm', '::elementSave'],
          ],
        ],
      ],
      '#prefix' => '<div class="ui-element-new">',
      '#suffix' => '</div>',
    ];

    $form['elements']['new']['region'] = [
      'data' => [],
    ];

    $form['elements']['new']['weight'] = [
      '#type' => 'weight',
      '#title' => $this->t('Weight for new element'),
      '#title_display' => 'invisible',
      '#default_value' => count($this->entity->getElements()) + 1,
      '#attributes' => ['class' => ['search-ui-element-order-weight']],
    ];
    $form['elements']['new']['operations'] = [
      'data' => [],
    ];

    return $form;
  }

  /**
   * Validate handler for search ui element.
   */
  public function elementValidate($form, FormStateInterface $form_state) {
    if (!$form_state->getValue('new')) {
      $form_state->setErrorByName('new', $this->t('Select an element to add.'));
    }
  }

  /**
   * Submit handler for search ui element.
   */
  public function elementSave($form, FormStateInterface $form_state) {
    $this->save($form, $form_state);

    // Check if this field has any configuration options.
    $element = $this->searchUiElementManager->getDefinition($form_state->getValue('new'));

    // Load the configuration form for this option.
    if (is_subclass_of($element['class'], '\Drupal\search_api_decoupled_ui\ConfigurableSearchUiElementInterface')) {
      $form_state->setRedirect(
        'search_api_decoupled.ui_element_add_form',
        [
          'search_api_endpoint' => $this->entity->id(),
          'ui_element' => $form_state->getValue('new'),
        ],
        ['query' => ['weight' => $form_state->getValue('weight')]]
      );
    }
    // If there's no form, immediately add the ui element.
    else {
      $element = [
        'id' => $element['id'],
        'settings' => [],
        'weight' => $form_state->getValue('weight'),
      ];
      $element_id = $this->entity->addElement($element);
      $this->entity->save();
      if (!empty($element_id)) {
        $this->messenger()->addStatus($this->t('The ui element was successfully added.'));
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);
    // Update search ui element weights.
    if (!$form_state->isValueEmpty('elements')) {
      $this->updateUiElementsWeights($form_state->getValue('elements'));
    }
    else {
      $form_state->setValue('elements', []);
    }
  }

  /**
   * Updates ui elements weights.
   *
   * @param array $elements
   *   Associative array with search ui elements having element uuid as keys and
   *   array with element settings as values.
   */
  protected function updateUiElementsWeights(array $elements) {
    $ui_element_regions = $this->entity->getUiElementRegions();
    $default_region = $this->entity->getDefaultUiElementRegion();
    foreach ($elements as $uuid => $element_data) {
      if ($this->entity->getElements()->has($uuid)) {
        $this->entity->getElement($uuid)->setWeight($element_data['weight']);
        $element_region = $this->entity->getElement($uuid)->getRegion();
        if (!in_array($element_region, array_keys($ui_element_regions))) {
          $this->entity->getElement($uuid)->setRegion($default_region);
        }
      }
    }
  }

}
