<?php

namespace Drupal\search_api_decoupled_ui\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Layout\LayoutPluginManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a block with decoupled search.
 *
 * @Block(
 *   id = "search_api_endpoint",
 *   admin_label = @Translation("Search API Endpoint"),
 *   category = @Translation("Search API Decoupled"),
 *   deriver = "Drupal\search_api_decoupled_ui\Plugin\Derivative\SearchApiEndpointBlock",
 * )
 */
class SearchApiEndpointBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The layout plugin manager.
   *
   * @var \Drupal\Core\Layout\LayoutPluginManagerInterface
   */
  protected $layoutPluginManager;

  /**
   * Constructs a new SearchBlock object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Layout\LayoutPluginManagerInterface $layout_plugin_manager
   *   The layout plugin manager.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager, LayoutPluginManagerInterface $layout_plugin_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityTypeManager = $entity_type_manager;
    $this->layoutPluginManager = $layout_plugin_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('plugin.manager.core.layout')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    /** @var \Drupal\search_api_decoupled\SearchApiEndpointInterface $search_api_endpoint */
    $search_api_endpoint = $this->entityTypeManager->getStorage('search_api_endpoint')->load($this->getDerivativeId());
    $elements = $search_api_endpoint->getElements();
    $search_elements = [];
    $regions = [];
    $content = [];
    $ui_regions = $search_api_endpoint->getUiElementRegions();
    foreach (array_keys($ui_regions) as $region) {
      $regions[$region] = [];
      /** @var \Drupal\search_api_decoupled_ui\SearchUiElementInterface $element */
      foreach ($elements as $element) {
        if ($element->getRegion() == $region) {
          $regions[$region][$element->getWeight()] = $element->getUuid();
        }
        if (empty($search_elements[$element->getUuid()])) {
          $search_elements[$element->getUuid()] = $element->getSearchUiConfig();
        }
      }
    }
    $layout = $this->layoutPluginManager->getDefinition($search_api_endpoint->getLayout());
    $layout_regions = $layout->getRegions();
    foreach ($regions as $region => $elements) {
      $regions[$region] = array_values($elements);
      $content[$region] = ['#markup' => '<p class="visually-hidden">' . $this->t('Region @region', ['@region' => $layout_regions[$region]['label']]) . '</p>'];
    }
    $block_id = 'block-' . $this->getMachineNameSuggestion();
    $base_url = $search_api_endpoint->getBaseUrl()->toString(TRUE);
    $build = $content + [
      '#theme' => $layout->getThemeHook(),
      '#layout' => $layout,
      '#attached' => [
        'library' => [
          'search_api_decoupled_ui/client',
          $layout->getLibrary(),
        ],
        'drupalSettings' => [
          'search_api_endpoint' => [
            $block_id => [
              'id' => $search_api_endpoint->id(),
              'label' => $search_api_endpoint->label(),
              'base_url' => $base_url->getGeneratedUrl(),
              'elements' => $search_elements,
              'layout' => [
                'id' => $search_api_endpoint->getLayout(),
                'regions' => $regions,
              ],
            ],
          ],
        ],
      ],
    ];
    $cacheable_metadata = new CacheableMetadata();
    $cacheable_metadata->addCacheableDependency($search_api_endpoint);
    $cacheable_metadata->addCacheableDependency($base_url);
    $cacheable_metadata->applyTo($build);
    return $build;
  }

}
